package classes;


import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import messif.buckets.Bucket;
import messif.buckets.BucketDispatcher;
import messif.buckets.BucketStorageException;
import messif.buckets.LocalBucket;
import messif.buckets.impl.MemoryStorageBucket;
import messif.buckets.split.impl.SplitPolicyVoronoiPartitioning;
import messif.objects.LocalAbstractObject;
import messif.objects.PrecompDistPerforatedArrayFilter;
import messif.objects.impl.MetaObjectSAPIRWeightedDist;
import messif.objects.util.AbstractObjectIterator;
import messif.objects.util.AbstractObjectList;
import messif.objects.util.StreamGenericAbstractObjectIterator;
import messif.pivotselection.AbstractPivotChooser;
import messif.pivotselection.KMeansPivotChooser;
import messif.statistics.Statistics;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 *
 * @author Vlastislav Dohnal (xdohnal), dohnal@fi.muni.cz, Faculty of Informatics, Masaryk University, Brno, Czech Republic
 */
public class ClusterByPivots {

    public static void main(String[] args) {

        try {
            // Disable statistics
            Statistics.disableGlobally();

            BucketDispatcher disp = new BucketDispatcher(Integer.MAX_VALUE, Long.MAX_VALUE, Long.MAX_VALUE, 0, true, MemoryStorageBucket.class);

            LocalBucket bucket = disp.createBucket();
            StreamGenericAbstractObjectIterator input = new StreamGenericAbstractObjectIterator(MetaObjectSAPIRWeightedDist.class, "datasets/sapir.nov-demo-10m/metaobject-100000-filter-abstractkey.data");

            bucket.addObjects(input);
            System.out.println("Objects read from the input stream: " + bucket.getObjectCount());

            // Initial pivots...
            StreamGenericAbstractObjectIterator inputPivots = new StreamGenericAbstractObjectIterator(MetaObjectSAPIRWeightedDist.class, "datasets/sapir.nov-demo-10m/queryset-50-nofilter-noskipgraphs.data");
            AbstractObjectList<LocalAbstractObject> initPivots = new AbstractObjectList<LocalAbstractObject>();
            while (inputPivots.hasNext())
                initPivots.add(inputPivots.next());
        
            // Set pivot chooser
            AbstractPivotChooser chooser = new KMeansPivotChooser(initPivots);
            chooser.registerSampleProvider(bucket);

            final int pivotCnt = initPivots.size(); //200;
            chooser.selectPivot(pivotCnt);
            LocalAbstractObject[] pivots = new LocalAbstractObject[pivotCnt];
            for (int p = 0; p < pivotCnt; p++)
                pivots[p] = chooser.getPivot(p);

            List<Bucket> resBuckets = new ArrayList<Bucket>();
            resBuckets.add(bucket);

            SplitPolicyVoronoiPartitioning policy = new SplitPolicyVoronoiPartitioning();
            policy.setPivots(pivots);

            System.out.println("Objects before splitting: " + bucket.getObjectCount());
            bucket.split(policy, resBuckets, disp, 0);        // This CAST is checked!!!
            long cnt = 0;
            for (Bucket b : resBuckets)
                cnt += ((LocalBucket)b).getObjectCount();
            System.out.println("Objects after splitting: " + cnt);

            BufferedOutputStream output = new BufferedOutputStream(new FileOutputStream("datasets/sapir.nov-demo-10m/metaobject-100000-clustered.data"));
            // Write all data out but bucket by bucket
            for (Bucket b : resBuckets) {
                if (b == null)
                    continue;
                AbstractObjectIterator<LocalAbstractObject> iter = b.getAllObjects();
                while (iter.hasNext()) {
                    LocalAbstractObject o = iter.next();

                    PrecompDistPerforatedArrayFilter filter = o.getDistanceFilter(PrecompDistPerforatedArrayFilter.class);
                    if (filter != null)
                        o.unchainFilter(filter);
                    
                    o.write(output);
                }
            }
            output.close();

        } catch (BucketStorageException ex) {
            Logger.getLogger(ClusterByPivots.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalArgumentException ex) {
            Logger.getLogger(ClusterByPivots.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(ClusterByPivots.class.getName()).log(Level.SEVERE, null, ex);
        }

        //bucket.a
    }
}
