package classes;

/*
 * DistanceDistribution.java
 *
 * Created on 16. unor 2007, 11:05
 */

import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import messif.objects.LocalAbstractObject;
import messif.objects.impl.MetaObjectMap;
import messif.objects.util.StreamGenericAbstractObjectIterator;
import messif.utility.Convert;

/**
 * Utility for computing distance between two (or more) objects in different files.
 *
 * @author xbatko
 */
public class GetDistance {

    public static void printUsage(PrintStream stream) {
        stream.println("Usage: GetDistance <class> <data file1> [<data file2 ...>]");
    }

    public static <T extends LocalAbstractObject> List<T> getObjects(Class<? extends T> objectClass, String[] files, int index) throws IOException {
        List<T> objects = new ArrayList<T>();
        for (; index < files.length; index++) {
            // Read all objects from the file
            StreamGenericAbstractObjectIterator<T> iterator = new StreamGenericAbstractObjectIterator<T>(objectClass, files[index]);
            while (iterator.hasNext())
                objects.add(iterator.next());
            iterator.close();
        }
        return objects;
    }

    public static void printDistance(LocalAbstractObject obj1, LocalAbstractObject obj2, PrintStream printStream) {
        if (obj1 instanceof MetaObjectMap && obj2 instanceof MetaObjectMap) {
            Map<String, LocalAbstractObject> obj1Map = ((MetaObjectMap)obj1).getObjectMap();
            Map<String, LocalAbstractObject> obj2Map = ((MetaObjectMap)obj2).getObjectMap();
            for (Map.Entry<String, LocalAbstractObject> entry : obj1Map.entrySet()) {
                LocalAbstractObject obj2object = obj2Map.get(entry.getKey());
                if (obj2object != null) {
                    printStream.print(" ");
                    printStream.print(entry.getValue().getDistance(obj2object));
                    printStream.print(" [");
                    printStream.print(entry.getKey());
                    printStream.print("]");
                }
            }
        } else {
            printStream.print(" ");
            printStream.print(obj1.getDistance(obj2));
        }
    }

    public static void main(String[] args) {
        
        if (args.length < 2) {
            printUsage(System.err);
            return;
        }

        try {
            Class<LocalAbstractObject> objectClass = Convert.getClassForName(args[0], LocalAbstractObject.class);
            List<LocalAbstractObject> objects = getObjects(objectClass, args, 1);

            if (objects.size() < 2) {
                System.err.println("Not enough objects loaded: " + objects.size());
                return;
            }

            // Compute distance
            LocalAbstractObject firstObj = objects.remove(0);
            for (LocalAbstractObject object : objects) {
                System.out.print(firstObj.getLocatorURI());
                System.out.print(" to ");
                System.out.print(object.getLocatorURI());
                System.out.print(":");
                printDistance(firstObj, object, System.out);
                System.out.println();
            }
        } catch (Exception e) {
            System.err.println(e.toString());
        }
    }
}
