package classes;


import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.HashSet;
import java.util.NoSuchElementException;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import messif.objects.util.AbstractObjectIterator;
import messif.objects.LocalAbstractObject;
import messif.objects.util.StreamGenericAbstractObjectIterator;
import messif.utility.Convert;

/**
 *
 * @author xbatko
 */
public class PickSelectedObjects {

    protected static void printUsage() {
        // Wrong argument count
        System.err.println("Usage: PickSelectedObjects <objects' class> [-in <input object file>] [-out <output object file>] {-ids|-pos} <locator id or position (zero based)> ...");
        System.err.println("       The objects are appended to the output file. If the output file does not exist, it is created.");
        System.err.println("       If the input file is not specified, System.in is used.");
        System.err.println("       If the output file is not specified, System.out is used.");
    }

    public static void main(String... args) throws Exception {
        try {
            // Parse arguments and open input and output files
            int argIndex = 1;

            // Objects class
            Class<LocalAbstractObject> objectsClass = Convert.getClassForName(args[0], LocalAbstractObject.class);

            // Object input file
            AbstractObjectIterator<LocalAbstractObject> input;
            if (args[argIndex].equals("-in")) {
                input = new StreamGenericAbstractObjectIterator<LocalAbstractObject>(objectsClass, args[argIndex + 1]);
                argIndex += 2;
            } else {
                input = new StreamGenericAbstractObjectIterator<LocalAbstractObject>(objectsClass, new BufferedReader(new InputStreamReader(System.in)));
            }

            // Object output file
            OutputStream output;
            if (args[argIndex].equals("-out")) {
                output = new FileOutputStream(args[argIndex + 1], true);
                argIndex += 2;
            } else {
                output = System.out;
            }

            // Process files
            if (args[argIndex].equals("-pos")) {
                // Get all positions from cmd line and sort them
                SortedSet<Integer> positions = new TreeSet<Integer>();
                for (argIndex++; argIndex < args.length; argIndex++)
                    positions.add(Integer.parseInt(args[argIndex])+1);
                
                // Write the objects out
                int lastPos = 0;
                try {
                    for (int pos : positions) {
                        pos -= lastPos;
                        lastPos += pos;
                        input.getObjectByPosition(pos).write(output);
                    }
                } catch (NoSuchElementException e) {
                    // There was additional objects that was not found
                    System.err.print("Positions not found: ");
                    System.err.println(positions.tailSet(lastPos + 1));
                }
            } else if (args[argIndex].equals("-ids")) {
                // Get all locatorURIs from cmd line
                Set<String> locatorURIs = new HashSet<String>();
                for (argIndex++; argIndex < args.length; argIndex++)
                    locatorURIs.add(args[argIndex]);
                
                // Write the objects out
                try {
                    while (!locatorURIs.isEmpty())
                        input.getObjectByAnyLocator(locatorURIs, true).write(output);
                } catch (NoSuchElementException e) {
                    // There was additional objects that was not found
                    System.err.print("IDs not found: ");
                    System.err.println(locatorURIs);
                }
            } else throw new IndexOutOfBoundsException("Either -ids or -pos must be specified");
        } catch (IndexOutOfBoundsException e) {
            // Wrong argument count
            printUsage();
        }
    }
}
